#uses "fwUnitTestComponentAsserts.ctl"
#uses "unSystemIntegrity.ctl"
#uses "unSystemIntegrity/unSystemIntegrity_alarmDescription.ctl"

const string TEST_ALARM_NAME = "testAlarmDescription";
const string TEST_DP_NAME = c_unSystemAlarm_dpPattern + TEST_ALARM_NAME;
const string TEST_DPE_NAME = TEST_DP_NAME + ".alarm";
const string TEST_DESCRIPTION = "Test text for alarm description";

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unSystemIntegrity_alarmDescription_alwaysRunInUI() {
}

//------------------------------------------------------
void test_unSystemIntegrity_alarmDescription_setupSuite() {
	if (dpExists(TEST_DP_NAME)) {
		dpDelete(TEST_DP_NAME);
	}

	// Create system integrity alarm
	dyn_string exceptionInfo;
	unSystemIntegrity_createSystemAlarm(TEST_ALARM_NAME, "", "test alarm", exceptionInfo);
	assertTrue(dynlen(exceptionInfo) == 0, "Alarm has not been properly created. Exception message: " + exceptionInfo);

	// Initialize the alarm
	dpSetWait(TEST_DP_NAME + ".enabled", true);
	unAlarmConfig_mask(TEST_DPE_NAME, false, exceptionInfo);
	assertTrue(dynlen(exceptionInfo) == 0, "Alarm has not been properly initiated. Exception message: " + exceptionInfo);
}


//------------------------------------------------------
void test_unSystemIntegrity_alarmDescription_handleAlarmDescription() {
 dyn_string exceptionInfo;
 shared_ptr<unSystemIntegrity_UpdateAlarmDescription> updAlDesc = new unSystemIntegrity_UpdateAlarmDescription();

 // register test alarm description and connect to CB function
 unSystemIntegrity_registerAlarmDescription(TEST_DPE_NAME, updAlDesc, exceptionInfo);
 classConnect("_unSystemIntegrity_test_alarmDescriptionCB", updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);

	// Set the alarm value to 10, Bad status
	int rc = dpSetWait(TEST_DPE_NAME, c_unSystemIntegrity_alarm_value_level1);
	assertTrue(dynlen(getLastError()) == 0, "Alarm has not been raised to Bad status");

 // Delay needed to offset the processing time of creating/updating the alarm description in the unit test
 delay(0, 500);

 // Obtain the System Integrity Alarm Description
	string alarmDescription = unSystemIntegrity_getAlarmDescription(TEST_DPE_NAME, exceptionInfo);
	
 assertTrue(dynlen(exceptionInfo) == 0, "Alarm description has returned the exception: " + exceptionInfo);
	assertEqual(TEST_DESCRIPTION, alarmDescription, "The system integrity alarm description does not match with what was expected");
 
 // unregister alarm description
 classDisconnect("_unSystemIntegrity_test_alarmDescriptionCB", updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);
}

//------------------------------------------------------
void test_unSystemIntegrity_alarmDescription_teardownSuite() {
	// Remove the test alarm datapoint
	if (dpExists(TEST_DP_NAME)) {
		dpDelete(TEST_DP_NAME);
	}
}

//-------------------------------------------------------
private void _unSystemIntegrity_test_alarmDescriptionCB(time timeStamp, atime aTimeStamp, string dpeAlert, int alarmValue) {
  unSystemIntegrity_setAlarmDescription(TEST_DESCRIPTION, timeStamp, aTimeStamp, dpeAlert);
}


