#uses "fwUnitTestComponentAsserts.ctl" 
#uses "unSystemIntegrity_unManagerState.ctl"

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_setupSuite(){
  // Stop the running manual managers before starting the unit tests
  _unSystemIntegrity_unManagerState_stopManualManagers();

  if (!dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern)) {
	unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_ADD,
                                                   unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
  }
  assertEqual(TRUE, dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern), "The alarm to monitor if a manager is running in manual mode has not been added.");
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_deleteAlarm(){
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_DELETE,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
  assertEqual(FALSE, dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern), "The alarm to monitor if a manager is running in manual mode has not been deleted.");

  // Add alarm again to continue with subsequent tests
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_ADD,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_enableAlarm(){
  bool alarmEnabled;
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_ENABLE,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
  int rc = dpGet(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".enabled", alarmEnabled);
  assertEqual(0, rc, "Alarm enabled datapoint cannot be obtained to check if alarm is enabled.");
  assertEqual(TRUE, alarmEnabled, "The alarm to monitor if a manager is running in manual mode has not been enabled.");
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_disableAlarm(){
  bool alarmEnabled;
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_DISABLE,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
  int rc = dpGet(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".enabled", alarmEnabled);
  assertEqual(0, rc, "Alarm enabled datapoint cannot be obtained to check if alarm is disabled.");
  assertEqual(FALSE, alarmEnabled, "The alarm to monitor if a manager is running in manual mode has not been disabled.");

  // Enable alarm again to continue with subsequent tests
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_ENABLE,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_alwaysRunning(){
  _unSystemIntegrity_unManagerState_addManager("WCCILsim", "-num 51", "always");
  _unSystemIntegrity_unManagerState_addManager("WCCOActrl", "libs/test/unSystemIntegrity/_unSystemIntegrity_unManagerState_testCtrlManager.ctl -num 52", "always");

  int iErrorCode;
  string dpe = c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".alarm";
  int rc = dpGet(dpe, iErrorCode);
  assertEqual(0, rc, "Error code cannot be obtained from the " + dpe + " alarm datapoint to check if manager is always running.");
  assertEqual(0, iErrorCode, "The iErrorCode should have been 0 as the managers are running always");

  _unSystemIntegrity_unManagerState_removeManagers();
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_onceRunning(){
  _unSystemIntegrity_unManagerState_addManager("WCCILsim", "-num 51", "once");
  _unSystemIntegrity_unManagerState_addManager("WCCOActrl", "libs/test/unSystemIntegrity/_unSystemIntegrity_unManagerState_testCtrlManager.ctl -num 52", "once");

  int iErrorCode;
  string dpe = c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".alarm";
  int rc = dpGet(dpe, iErrorCode);
  assertEqual(0, rc, "Error code cannot be obtained from the " + dpe + " alarm datapoint to check if manager is running once.");
  assertEqual(0, iErrorCode, "The iErrorCode should have been 0 as the managers are running once");

  _unSystemIntegrity_unManagerState_removeManagers();
}

//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_manualRunning(){
  _unSystemIntegrity_unManagerState_addManager("WCCILsim", "-num 51", "manual");
  _unSystemIntegrity_unManagerState_addManager("WCCOActrl", "libs/test/unSystemIntegrity/_unSystemIntegrity_unManagerState_testCtrlManager.ctl -num 52", "manual");

  int iErrorCode;
  int count;
  string dpe = c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".alarm";
  int rc = dpGet(dpe, iErrorCode);
  assertEqual(0, rc, "Error code cannot be obtained from the " + dpe + " alarm datapoint to check if manager is running in manual mode.");
  assertEqual(10, iErrorCode, "The iErrorCode should have been 10 as the managers are running in manual mode");

  time t;
  string sManagerRunningInManual;
  dyn_string dt, dsManagerRunningInManual;
  rc = dpGet(dpe + ":_alert_hdl.._alerts", dt);
  assertEqual(0, rc, "Time from the alert datapoint for " + dpe + " cannot be obtained.");
  t = dt[dynlen(dt)];
  atime at = makeATime(t, 0, dpe + ":_alert_hdl.2._comment");

  do {
    delay(5);
    rc = alertGet(t, getACount(at), dpSubStr(getAIdentifier(at), DPSUB_SYS_DP_EL_CONF_DET) + "._comment", sManagerRunningInManual);
    assertEqual(0, rc, "The managers running in manual mode cannot be obtained from the alertGet function.");
    dsManagerRunningInManual = strsplit(sManagerRunningInManual, "|");
    count++;
  } while(dynlen(dsManagerRunningInManual) != 2 && count < 12);

  assertTrue(dynlen(dsManagerRunningInManual) >= 2, "Number of managers running in manual mode should have been at least 2");

  dsManagerRunningInManual[1].trim();
  dsManagerRunningInManual[2].trim();
  assertEqual("Manager: WCCILsim num 51", dsManagerRunningInManual[1], "The manager WCCILsim num 51 should have been retrieved from the alert");
  assertEqual("Manager: WCCOActrl num 52", dsManagerRunningInManual[2], "The manager WCCOActrl num 52 should have been retrieved from the alert");

  _unSystemIntegrity_unManagerState_removeManagers();
}


//------------------------------------------------------------------
void test_unSystemIntegrity_unManagerState_teardownSuite(){
  // Add managers again in case all the tests passed, so there is not an error invoked when removing the managers.
  // *Note: Removing the managers from teardown is necessary in case one of the tests failed.
  fwInstallationManager_add("WCCILsim", "manual", 2, 2, 30, "-num 51");
  fwInstallationManager_add("WCCOActrl", "manual", 2, 2, 30, "libs/test/unSystemIntegrity/_unSystemIntegrity_unManagerState_testCtrlManager.ctl -num 52");

  _unSystemIntegrity_unManagerState_removeManagers();
  unSystemIntegrity_unManagerState_HandleCommand(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".command:_online.._value", UN_SYSTEMINTEGRITY_DISABLE,
                                                 unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.parameters:_online.._value", monitorManualRunning_pattern);
}

//------------------------------------------------------------------
private void _unSystemIntegrity_unManagerState_stopManualManagers(){
  string sManagerType, sManagerOption;
  int iManagerState, iManagerMode;
  for (int i=0; i<pmonGetCount(); i++) {
	// Get the manager type, option, state, and mode
	sManagerType = pmonGetName(i);
	sManagerOption = pmonGetOptions(i);
	iManagerState = pmonGetState(i);
	iManagerMode = pmonGetStartMode(i);
	if (iManagerMode == 0 && iManagerState == 2 && sManagerType != "WCCOAui")
		fwInstallationManager_command("KILL", sManagerType, sManagerOption);
  }
}

//------------------------------------------------------------------
private void _unSystemIntegrity_unManagerState_removeManagers(){
  _unSystemIntegrity_unManagerState_deleteManager("WCCOActrl", "libs/test/unSystemIntegrity/_unSystemIntegrity_unManagerState_testCtrlManager.ctl -num 52", "manual");
  _unSystemIntegrity_unManagerState_deleteManager("WCCILsim", "-num 51", "manual");
}

//------------------------------------------------------------------
private void _unSystemIntegrity_unManagerState_addManager(string managerType, string managerOption, string managerMode) {
  int count;
  fwInstallationManager_add(managerType, managerMode, 2, 2, 30, managerOption);
  fwInstallationManager_command("START", managerType, managerOption);

  bool isRunning;
  fwInstallationManager_isRunning(managerType, managerOption, isRunning);
  while(!isRunning && count < 12) {
	delay(5);
	fwInstallationManager_isRunning(managerType, managerOption, isRunning);
	count++;
  }
}

//------------------------------------------------------------------
private void _unSystemIntegrity_unManagerState_deleteManager(string managerType, string managerOption, string managerMode){
  int count;
  fwInstallationManager_setMode(managerType, managerOption, managerMode);
  fwInstallationManager_command("KILL", managerType, managerOption);

  bool isStopped;
  fwInstallationManager_isStopped(managerType, managerOption, isStopped);
  while(!isStopped && count < 12) {
	delay(5);
	fwInstallationManager_isStopped(managerType, managerOption, isStopped);
	count++;
  }

  fwInstallationManager_remove(managerType, managerOption);
}
